﻿using OpenBveApi.Runtime;

namespace OpenbveFcmbTrainPlugin
{
    /// <summary>A class for displaying messages to the user.</summary>
    internal static class MessageManager
    {
        /// <summary>The callback function to show an interface message.</summary>
        private static AddInterfaceMessageDelegate MessageDelegate;

        /// <summary>The callback function to show a score message.</summary>
        private static AddScoreDelegate ScoreDelegate;

        /// <summary>Initializes the message manager.</summary>
        /// <param name="messageDelegate">The delegate to add interface messages.</param>
        /// <param name="scoreDelegate">The delegate to add score messages.</param>
        internal static void Initialize(AddInterfaceMessageDelegate messageDelegate, AddScoreDelegate scoreDelegate)
        {
            MessageDelegate = messageDelegate;
            ScoreDelegate = scoreDelegate;
        }

        /// <summary>Shows a message to the player.</summary>
        /// <param name="message">The message to be shown.</param>
        /// <param name="color">The color of the message.</param>
        /// <param name="time">The time to show the message, in seconds.</param>
        internal static void ShowMessage(string message, OpenBveApi.Colors.MessageColor color, double time)
        {
            MessageDelegate(message, color, time);
        }

        /// <summary>Shows a message to the player.</summary>
        /// <param name="message">The message to be shown.</param>
        /// <param name="color">The color of the message.</param>
        /// <param name="time">The time to show the message, in seconds.</param>
        /// <param name="score">The score for the player.</param>
        internal static void ShowMessage( string message, OpenBveApi.Colors.MessageColor color, double time, int score)
        {
            ScoreDelegate(score, message, color, time);
        }
    }
}
